import java.util.*;
import java.lang.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
public class Experiment{

	static boolean witness(int a, int n){
		int t = 0;
		int u = n - 1;
		while(u % 2 == 0){
			u = u/2;
			t += 1;
		}
		int[] x = new int[t+1];
		x[0] = expMod(a, u, n);
		// System.out.println("t = " + t + ", u = " + u);

		for(int i = 1; i <= t; ++i){
			x[i] = expMod(x[i - 1], 2, n);
			if(x[i] < 0){
				x[i] += n;
			}
			// System.out.println("x[" + (i-1) + "] = " + x[i-1] + ", x[" + i + "] = " + x[i]);
			if(x[i] == 1 && x[i-1] != 1 && x[i-1] != n-1)
				return true;
		}
		if(x[t] != 1){
			return true;
		}else{
			return false;
		}
	}

	static boolean isPrime(int n, int s){
		Random rand = new Random();

		for(int a, j = 1; j <= s; ++j){
			a = rand.nextInt(n-1) + 1; //random(1, n-1)
			if(witness(a, n)){
				// System.out.println("Witness failed" + a + ", " + n);
				return false;
			}
		}
		return true;
	}

	static int expMod(int x, int y, int p){
		if (x == 0)
			return 0;
		if (y == 0)
			return 1;

		long tmp;
		if (y % 2 == 0){
			tmp = expMod(x, y / 2, p);
			tmp = (tmp * tmp) % p;
		}else{
			tmp = x % p;
			tmp = (tmp * expMod(x, y - 1, p) % p) % p;
		}

		return (int)((tmp + p) % p);
	}

	static int pow(int base, int power){
		if(power == 0)
			return 1;
		return base * pow(base, power - 1);
	}

	static String bin(int number){
		int[] binary = new int[32]; //Java int max 2 ^ 32
		int index = 0;
		while(number > 0){
			binary[index++] = number % 2;
			number = number / 2;
		}
		StringBuilder sbf = new StringBuilder();
		for(int i = index - 1; i >= 0; i--){
			sbf.append(binary[i]);
		}
		return sbf.toString();

	}

	public static void main(String[] args) {
		int start = 1000000;
		int end   = 2000000;
		int num_trials = 10;
		int bin_size = 10000;
		int bin_num = (end - start) / bin_size;
		int[] bins = new int[bin_num];
		Arrays.fill(bins, 0);
		for(int i = start; i < end; i++){
			if(isPrime(i, num_trials)){
				if(isPrime(i+2, num_trials)){
					bins[(i-start) / bin_size] += 1;
				}
			}
		}
		try (PrintWriter writer = new PrintWriter("result" + num_trials + ".csv")) {

			StringBuilder sb = new StringBuilder();
			sb.append("bins,prime,nDividedByLognSquared,predict\n");
			for(int i = 0; i < bin_num; i++){
				sb.append((start + i*bin_size));
				sb.append(",");
				sb.append(bins[i]);
				sb.append(",");
				double x = (double)(start + i*bin_size)/(Math.log((double)(start + i*bin_size)) * Math.log((double)(start + i*bin_size)));
				x -= (double)(start + (i-1)*bin_size)/(Math.log((double)(start + (i-1)*bin_size)) * Math.log((double)(start + (i-1)*bin_size)));
				sb.append(x);
				sb.append(",0\n");
			}

			writer.write(sb.toString());

			System.out.println("result" + num_trials + ".csv done!");

		} catch (FileNotFoundException e) {
			System.out.println(e.getMessage());
		}
	}
}